//
// C-style include file that defines all the externally-accessible UUDeview
//  functions
//

/*
 * This file is part of uudeview, the simple and friendly multi-part multi-
 * file uudecoder  program  (c)  1994 by Frank Pilhofer. The author may be
 * contacted by his email address,          fp@informatik.uni-frankfurt.de
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef __UUDEVIEW_H__
#define __UUDEVIEW_H__

/*
 * Message Types
 */

#define UUMSG_MESSAGE	(0)	/* just a message, nothing important */
#define UUMSG_NOTE	(1)	/* something that should be noticed */
#define UUMSG_WARNING	(2)	/* important msg, processing continues */
#define UUMSG_ERROR	(3)	/* processing has been terminated */
#define UUMSG_FATAL	(4)	/* decoder cannot process further requests */
#define UUMSG_PANIC	(5)	/* recovery impossible, app must terminate */

/*
 * Return Values
 */

#define UURET_OK	(0)	/* everything went fine */
#define UURET_IOERR	(1)	/* I/O Error - examine errno */
#define UURET_NOMEM	(2)	/* not enough memory */
#define UURET_ILLVAL	(3)	/* illegal value for operation */
#define UURET_NODATA	(4)	/* decoder didn't find any data */
#define UURET_NOEND	(5)	/* encoded data wasn't ended properly */
#define UURET_UNSUP	(6)	/* unsupported function (encoding) */
#define UURET_EXISTS	(7)	/* file exists (decoding) */
#define UURET_CONT	(8)	/* continue -- special from ScanPart */
#define UURET_CANCEL	(9)	/* operation canceled */

/*
 * File states, may be OR'ed
 */

#define UUFILE_READ	(0)	/* Read in, but not further processed */
#define UUFILE_MISPART	(1)	/* Missing Part(s) detected */
#define UUFILE_NOBEGIN	(2)	/* No 'begin' found */
#define UUFILE_NOEND	(4)	/* No 'end' found */
#define UUFILE_NODATA	(8)	/* File does not contain valid uudata */
#define UUFILE_OK	(16)	/* All Parts found, ready to decode */
#define UUFILE_ERROR	(32)	/* Error while decoding */
#define UUFILE_DECODED	(64)	/* Successfully decoded */
#define UUFILE_TMPFILE	(128)	/* Temporary decoded file exists */

/*
 * Encoding Types
 */

#define UU_ENCODED	(1)	/* UUencoded data   */
#define B64ENCODED	(2)	/* Mime-Base64 data */
#define XX_ENCODED	(3)	/* XXencoded data   */
#define BH_ENCODED	(4)	/* Binhex encoded   */
#define PT_ENCODED	(5)	/* Plain-Text encoded (MIME) */
#define QP_ENCODED	(6)	/* Quoted-Printable (MIME)   */

/*
 * Option indices for GetOption / SetOption
 */

#define UUOPT_VERSION	(0)	/* version number MAJOR.MINORplPATCH (ro) */
#define UUOPT_FAST	(1)	/* assumes only one part per file */
#define UUOPT_DUMBNESS	(2)	/* switch off the program's intelligence */
#define UUOPT_BRACKPOL	(3)	/* give numbers in [] higher precendence */
#define UUOPT_VERBOSE	(4)	/* generate informative messages */
#define UUOPT_DESPERATE	(5)	/* try to decode incomplete files */
#define UUOPT_IGNREPLY	(6)	/* ignore RE:plies (off by default) */
#define UUOPT_OVERWRITE	(7)	/* whether it's OK to overwrite ex. files */
#define UUOPT_SAVEPATH	(8)	/* prefix to save-files on disk */
#define UUOPT_IGNMODE	(9)	/* ignore the original file mode */
#define UUOPT_DEBUG	(10)	/* print messages with FILE/LINE info */
#define UUOPT_ERRNO	(14)	/* get last error code for UURET_IOERR (ro) */
#define UUOPT_PROGRESS	(15)	/* retrieve progress information */
#define UUOPT_USETEXT	(16)	/* handle text messages */
#define UUOPT_PREAMB	(17)	/* handle Mime preambles/epilogues */
#define UUOPT_TINYB64	(18)	/* detect short B64 outside of Mime */
#define UUOPT_ENCEXT	(19)	/* extension for single-part encoded files */
#define UUOPT_REMOVE    (20)    /* remove input files after decoding */
#define UUOPT_MOREMIME  (21)    /* strict MIME adherence */


/*
 * Code for the "action" in the progress structure
 */

#define UUACT_IDLE	(0)	/* we don't do anything */
#define UUACT_SCANNING	(1)	/* scanning an input file */
#define UUACT_DECODING	(2)	/* decoding into a temp file */
#define UUACT_COPYING	(3)	/* copying temp to target */
#define UUACT_ENCODING	(4)	/* encoding a file */


/*
 * Structure that holds the information for a single file / part of
 * a file. If a subject line is encountered, it is copied to subject;
 * if a begin is found, the mode and name of the file is extracted.
 * flags are set if 'begin' or 'end' is detected and 'uudet' if valid
 * uuencoded data is found. If the file contains a 'From:' line with
 * a '@' in it (indicating an origin email address), it is preserved
 * in 'origin'.
 **/

typedef struct _fileread {
  char *subject;		/* Whole subject line */
  char *filename;		/* Only filled in if begin detected */
  char *origin;			/* Whole 'From:' line */
  char *mimeid;			/* the ID for Mime-encoded files */
  char *mimetype;		/* Content-Type */
  short mode;			/* Mode of File (from 'begin') */
  int	begin;			/* begin detected */
  int	end;			/* end detected */
  int	flags;			/* associated flags */

  short uudet;			/* valid encoded data. value indicates encoding */
  short partno;			/* Mime-files have a part number within */
  short maxpno;			/* ... plus the total number of parts	*/

  char *sfname;			/* Associated source file */
  long startpos;		/* ftell() position where data starts */
  long length;			/* length of data */
} fileread;


/*
 * Structure for holding one part of a file, with some more information
 * about it. The UUPreProcessPart() function takes one a fileread structure
 * and produces this uufile structure.
 * Linked List, ordered by partno.
 **/

typedef struct _uufile {
  char	   *filename;
  char	   *subfname;
  char	   *mimeid;
  char	   *mimetype;
  short		partno;
  fileread *data;
  struct _uufile *NEXT;
} uufile;


/*
 * Structure for holding the list of files that have been found
 * uufile items are inserted into this list with UUInsertPartToList
 * After inserting a bunch of files, UUCheckGlobalList must be called
 * to update the states.
 */

typedef struct _uulist {
  short    state;		/* Status as described by the macros above */
  short    mode;		/* file mode as found on begin line        */

  int      begin;		/* part number where begin was detected    */
  int      end;			/* part number where end was detected      */

  short    uudet;		/* Encoding type (see macros above)        */
  int      flags;		/* flags, especially for single-part files */

  long     size;		/* approximate size of resulting file      */
  char    *filename;		/* malloc'ed file name                     */
  char    *subfname;		/* malloc'ed ID from subject line          */
  char    *mimeid;		/* malloc'ed MIME-ID, if available         */
  char    *mimetype;		/* malloc'ed Content-Type, if available    */

  char    *binfile;		/* name of temp file, if already decoded   */

  struct _uufile *thisfile;	/* linked list of this file's parts        */

  int     *haveparts;		/* the parts we have (max. 256 are listed) */
  int     *misparts;		/* list of missing parts (max. 256)        */

  struct _uulist *NEXT;		/* next item of the list                   */
  struct _uulist *PREV;		/* previous item of the list               */
} uulist;


/*
 * The "progress" structure which is passed to the Busy Callback
 */

typedef struct {
  int  action;			/* see UUACT_* definitions above           */
  char curfile[256];		/* the file we are working on, incl. path  */
  int  partno;			/* part we're currently decoding           */
  int  numparts;		/* total number of parts of this file      */
  long fsize;			/* size of the current file                */
  int  percent;			/* % of _current part_                     */
  long foffset;			/* file offset -- internal use only        */
  long totsize;			/* file total size -- internal use only    */
} uuprogress;


/*
 * Externally visible Functions
 */
#if defined(_WIN32) || defined(WIN32)
#define UUEXPORT _stdcall
#else
#define UUEXPORT _export _far _pascal
#endif


#ifdef __cplusplus
extern "C" {
#endif

// First-time setup
extern int UUEXPORT UUInitialize(void);

// Retrieve options
extern int UUEXPORT					// Returns integer option value, 0 for strings, -1 for error
UUGetOption(int option_no,			// Option to retrieve
		    int *int_return,		// Integer Return buffer
			char *string_return,	// String Return buffer
		    int string_return_len);	// Length of String Return buffer

// Set options
extern int UUEXPORT					// Returns UURET_ILLVAL for illegal option, UURET_OK otherwise
UUSetOption(int option_no,			// Option to set
			int int_value,			// Value for integer options
			char *string_value);	// Value for string options

// Converts a UU error code to a string
extern char * UUEXPORT				// Error string
UUstrerror(int code);				// Code to convert

// Sets a function that is called when the UU library wants to send
// a message to the caller
extern int UUEXPORT					// Returns UURET_OK
UUSetMsgCallback(void *opaque,		// Any pointer; will be passed to callback
				 void (*callback) (void *opaque,	// Opaque pointer from SetMsgCallback
						   char *message,			// Message from UU library
						   int level));				// Severity of message

// Sets a function that will be called periodically during the
// decoding process
extern int UUEXPORT					// Returns UURET_OK
UUSetBusyCallback(void *opaque,		// Any pointer; will be passed to callback
				  int (*callback) (void *opaque, uuprogress *progress),	// Opaque pointer from Set
				  long interval);			// Number of milliseconds between calls

// Set a function that will be called before the UU library attempts to
// read each file.
extern int UUEXPORT					// Returns UURET_OK
UUSetFileCallback(void *opaque,		// Any pointer to be passed to callback function
				  int (*callback)			// Return UURET_OK if operation was successful
				      (void *opaque,		// Pointer from Set
					   char *remote_name,	// Name of file to retrieve/enable
					   char *local_name,	// "Local" name of file (copy remote_name if not different)
					   int));				// 1=Enable remote file, 0=Disable

// Set a function that will receive each decoded filename before the
// file is opened.  Filename can be altered to suit local system limitations.
extern int UUEXPORT					// Returns UURET_OK
UUSetFNameFilter(void *opaque,		// Any pointer, will be passed to filter function
				 char * (*filter)			// Return altered filename
				        (void *opaque,		// Pointer from Set
						 char *filename));	// Original malloc'd filename

// Run the current filename filter on a filename
extern char * UUEXPORT				// Returns filtered filename
UUFNameFilter(char *filename);		// Filename to filter

// Return a pointer to the nth item in the File List
extern uulist * UUEXPORT			// Returns pointer to the item or NULL if no such item
UUGetFileListItem(int n);			// Item to retrieve

// The first part of the decoding process.  Each file passed to this function
// is parsed.  If any decodable files are found, they will be added to the
// global File List.
extern int UUEXPORT					// Returns UURET_OK or error code
UULoadFile(char *filename,			// File to load
		   char *fileid,			// Identifying information about file (or NULL if none)
		   int delflag);			// Delete file after decoding (or on cleanup)

// Changes the planned output filename of a decodable file on the
// File List
extern int UUEXPORT					// Returns UURET_OK or error code
UURenameFile(uulist *fl_item,		// File List Item to alter
			 char *newname);		// New output filename


// Decode a file in the File List to a temporary file
extern int UUEXPORT					// Returns UURET_OK or error code
UUDecodeToTemp(uulist *fl_item);	// Item to decode

// Remove the temporary file created by UUDecodeToTemp
extern int UUEXPORT					// Returns UURET_OK or error code
UURemoveTemp(uulist *fl_item);

// Decodes a file in the File List to a temporary file,
// then moves it to a destination.  If the "save path" has been
// set in the Options, the destination file will be created using
// that path
extern int UUEXPORT					// Returns UURET_OK or error code
UUDecodeFile(uulist *fl_item,		// Item to decode
			 char *dest);			// Destination filename

// Retrieve information about a file in the File List.  This may include
// news headers, the "zeroth" part of a multipart file, etc.
// The callback function is sent lines of information until none remains
// or until it returns a non-zero value.
extern int UUEXPORT					// Returns UURET_OK or error code
UUInfoFile(uulist *fl_item,			// Item to examine
		   void *opaque,			// Any pointer; will be passed to callback function
		   int (*callback)			// Return 0 to continue reading info or NZ to stop
		       (void *opaque,		// Pointer passed to UUInfoFile
				char *infoline));	// One line of information about the file

extern int UUEXPORT UUSmerge(int);

// Delete the File List and clear the Save Path for another
// decoding operation
extern int UUEXPORT UUCleanUp(void);	// Returns UURET_OK

// Encode a binary file to a stream in one piece using MIME multipart
// headers
extern int UUEXPORT                 // UURET_OK or otherwise
UUEncodeMulti(FILE *outfile, 		// Output stream
			  FILE *infile,			// Input stream (NULL means open infname)
			  char *infname, 		// Input filename (if not stream)
			  int encoding,			// Encoding type
			  char *outfname, 		// Filename to encode "as" if not the same is infname
			  char *mimetype, 		// MIME content type
			  int filemode);		// UNIX file mode (==> 0)

// Encode a binary file to a stream in pieces with headers
extern int UUEXPORT                 // UURET_OK or otherwise
UUEncodePartial(FILE *outfile, 		// Output stream
				FILE *infile,		// Input stream (NULL means open infname)
		 		char *infname, 		// Input filename (if not stream)
		 		int encoding,       // Encoding type
		 		char *outfname, 	// Filename to encode "as" if not the same is infname
		 		char *mimetype,		// MIME content type
		 		int filemode, 		// UNIX file mode (==> 0)
		 		int partno, 		// Segment number
		 		long linperfile);	// Number of lines per file

// Encode a binary file to a stream -- no headers
extern int UUEXPORT 				// UURET_OK or otherwise
UUEncodeToStream(FILE *outfile, 	// Output stream
				 FILE *infile,		// Input stream (NULL means open infname)
		  		 char *infname, 	// Input filename (if not stream)
		  		 int encoding,		// Encoding type
		  		 char *outfname, 	// Filename to encode "as" if not the same is infname
		  		 int filemode);		// UNIX file mode (==> 0)

// Encode straight to a disk file(s), no headers
extern int UUEXPORT
UUEncodeToFile(FILE *infile, 		// Input stream (NULL means open infname)
			   char *infname, 		// Input filename (if not stream)
			   int encoding,		// Encoding type
			   char *outfname, 		// Filename to encode "as" if not the same is infname
			   char *diskname, 		// Name of output file
			   long linperfile);		// Number of lines per file (<= 0 means one file)

// Encode to a single file using Mail/News style headers
extern int UUEXPORT 				// UURET_OK or otherwise
UUE_PrepSingle (FILE *outfile, 		// Output stream
				FILE *infile,		// Input stream (NULL means open infname)
				char *infname, 		// Input file (if not stream)
				int encoding,		// Encoding type
				char *outfname, 	// Filename to encode "as" if not the same is infname
				int filemode,		// UNIX file mode (==> 0)
				char *destination, 	// Destination e-mail address or newsgroup
				char *from,			// Sender's name/email
				char *subject, 		// Subject line
				int isemail);		// NZ means e-mail; Z means newsgroup

// Encode to multiple files using Mail/News style headers
extern int UUEXPORT
UUE_PrepPartial(FILE *outfile, 		// Output stream
				FILE *infile,		// Input stream (NULL means open infname)
				char *infname, 		// Input file (if not stream)
				int encoding,		// Encoding type
				char *outfname, 	// Filename to encode "as" if not the same is infname
				int filemode,		// UNIX file mode (==> 0)
				int partno,			// Which part
				long linperfile, 	// Number of lines per file
				long filesize,      // Size of the file
				char *destination, 	// Destination e-mail address or newsgroup
				char *from,			// Sender's name/email
				char *subject, 		// Subject line
				int isemail);		// NZ means e-mail; Z means newsgroup
#ifdef __cplusplus
	}
#endif
#endif

